import unittest
import os
from baraqdalib.addresses import Addresses


class AddressesTestCase(unittest.TestCase):
    def test_sym(self):
        os.chdir('../')
        a = Addresses()
        self.assertEqual([a._get_sym_city('Piorunów'), a._get_sym_city('Regut'), a._get_sym_city('Tłuste'), a._get_sym_city('Brzozówka'),
                          a._get_sym_city('Siemierz Górny'), a._get_sym_city('Horyszów-Nowa Kolonia')],
                         ['307', '1063', '2536', '1146', '896829', '898030'])


if __name__ == '__main__':
    unittest.main()



from baraqdalib import Generator
import csv
import random
from typing import Dict
import pkgutil


class Addresses:
    """Class Addresses for generating polish fake address but with true distribution"""
    def __init__(self):
        self._postal_codes: Dict[str] = dict()
        self._streets: Dict[str] = dict()
        self._cities: Dict[Dict[str]] = dict(dict())

        self._set_cities()
        self._set_streets()
        self._set_postal_codes()

    def _set_cities(self):
        """Get cities names  from file cities.csv and save in self._cities dict

        Parameters : None

        Returns: None
        """
        cities_file = pkgutil.get_data(__package__, 'data/PL/cities.csv')
        cities_file = csv.DictReader(cities_file.decode('utf-8-sig').splitlines(), delimiter=';')

        for row in cities_file:
            city = {row['SYM']: row['NAZWA']}
            self._cities.update(city)

    def _set_streets(self):
        """Get street names from file streets.csv and save in self._streets dict

        Parameters : None

        Returns: None
        """
        streets_file = pkgutil.get_data(__package__, 'data/PL/streets.csv')
        streets_file = csv.DictReader(streets_file.decode('utf-8-sig').splitlines(), delimiter=';')

        for street in streets_file:
            streets = {street['SYM']: list()}
            self._streets.update(streets)

        streets_file = pkgutil.get_data(__package__, 'data/PL/streets.csv')
        streets_file = csv.DictReader(streets_file.decode('utf-8-sig').splitlines(), delimiter=';')

        for street in streets_file:
            self._streets[street['SYM']].append(str(street['CECHA']+' '+street['NAZWA_2']+' '+street['NAZWA_1']).replace('  ', ' '))

    def _set_postal_codes(self):
        """Get postal codes from file postal_codes.csv and save in self._postal_code dict

        Parameters : None

        Returns: None
        """
        postal_codes_file = pkgutil.get_data(__package__, 'data/PL/postal_codes.csv')
        postal_codes_file = csv.DictReader(postal_codes_file.decode('utf-8-sig').splitlines(), delimiter=';')

        for postal_code in postal_codes_file:
            city = {postal_code['MIEJSCOWOŚĆ']: list()}
            self._postal_codes.update(city)

        postal_codes_file = pkgutil.get_data(__package__, 'data/PL/postal_codes.csv')
        postal_codes_file = csv.DictReader(postal_codes_file.decode('utf-8-sig').splitlines(), delimiter=';')

        for postal_code in postal_codes_file:
            self._postal_codes[postal_code['MIEJSCOWOŚĆ']].append(postal_code['KOD POCZTOWY'])

    def generate(self, lang: str = 'PL'):
        """ Generating address with city, street, street number,  and postal code

        Parameters:
        lang (str): Default is 'PL'. By this argument you can choose from which country addresses should be generated.

        Returns:
        Dict(): returning generated address in dictionary for easy access

        """
        address_generator = Generator()
        city = str(address_generator.generate(lang, 'cities_pops', 1, '\t')[0])
        sym = self._get_sym_city(city)
        street = self._get_streets(str(sym))
        postal_code = self._get_postal_code(city)
        address = {'street': street, 'city': city, 'postal_code': postal_code}
        return address

    def _get_sym_city(self, city: str):
        """Searching for city in cities and returning sym (identifier)

        Parameters:
        city (str): name of city for searching

        Returns:
        str : return sym for searching city
        """
        return list(self._cities.keys())[list(self._cities.values()).index(city)]

    def _get_postal_code(self, city: str):
        """Search and return postal code for a given city

        Parameters:
        city (str): Name of the city

        Returns:
        str: Returns postal code
        """
        city.replace('-', ' ')
        return random.choice(self._postal_codes[city])

    def _get_streets(self, city_sym: str):
        """ Generate random street for given city sym (identifier) that is in this city.

        Parameters:
        city_sym (str): number of city syn

        Returns:
        str: returns random street
        """
        try:
            streets = self._streets[city_sym]
            return random.choice(streets) + ' ' + str((round(random.lognormvariate(1.6, 2)) + 1) % 200)
        except KeyError:
            rand_sym = random.choice(list(self._streets.keys()))
            streets = self._streets[rand_sym]
            return random.choice(streets) + ' ' + str((round(random.lognormvariate(1.6, 2)) + 1) % 200)



from baraqdalib import Generator
from typing import Dict, List
from random import randrange
import random
import pkgutil
import json
import csv


class Cars:
    """Class Cars for generating vehicle infromation with using of statistical data from Poland to ensure propability of occurrence"""
    def __init__(self):
        self._car_generator = Generator()
        self._car: Dict[int, Dict[str]] = dict(dict())
        self._models: Dict[str, List[List[str]]] = dict(list(list()))

        self._filenames: List[str] = ['data/PL/cars/add_info_of_au.csv', 'data/PL/cars/add_info_of_vw.csv',
                                      'data/PL/cars/add_info_of_mb.csv', 'data/PL/cars/add_info_of_bmw.csv']
        self._vin_makers: Dict[str, list[str]] = {'MERCEDES-BENZ': ['VSC', 'WDB', 'WDC', 'WDD', 'WMX', 'W1K', 'W1N'],
                                                  'AUDI': ['TRU', 'WAU', 'WUA'],
                                                  'BMW': ['WBA', 'WBS', 'WBX', 'WBY', 'WB5'],
                                                  'VOLKSWAGEN': ['WVG', 'WVW']}
        self._body_type: Dict[str, str] = {'KARETA (SEDAN)': 'SEDAN', 'KOMBI': 'STATION WAGON', 'TERENOWY': 'SUV', 'HATCHBACK': 'HATCHBACK', 'INNY': 'DIFFERENT'}

        self._get_add_info_of_cars()

    def _get_add_info_of_cars(self):
        """Loading files with model for each car brand

        Parameters: None

        Return: None
        """
        for filename in self._filenames:
            models_file = pkgutil.get_data(__package__, filename)
            models = csv.reader(models_file.decode('utf-8').splitlines(), delimiter='\t')
            iterator = 0
            maker = ''
            temp_maker_letters = filename.split('.')[0][-2:]
            if temp_maker_letters == 'au':
                maker = 'AUDI'
            if temp_maker_letters == 'vw':
                maker = 'VOLKSWAGEN'
            if temp_maker_letters == 'mb':
                maker = 'MERCEDES-BENZ'
            if temp_maker_letters == 'mw':
                maker = 'BMW'
            self._models[maker] = {}
            for model in models:
                self._models[maker][iterator] = model
                iterator += 1

    def _toss_a_model(self, automaker: str):
        """Toss a model from specific brand

        Parameters:
        automaker (str):  Name of brand to toss from models from this brand

        Return:
        list (str):  Return list of attributes of tossed model
        """
        return self._models[automaker][randrange(0, len(self._models))]

    def _get_vin(self, automaker: str, production_year: str):
        """Create Vehcile Identification Number (VIN)

        Parameters:
        automaker (str):  Name of brand
        production_year (str)

        Return:
        str:  Return generated VIN
        """
        letters = 'ABCDEFGHJKLMNPRSTUVWXYZ'
        letters_nb = 'ABCDEFGHJKLMNPRSTUVWXYZ0123456789'
        nb_for_letter = [1, 2, 3, 4, 5, 6, 7, 8, 1, 2, 3, 4, 5, 7, 9, 2, 3, 4, 5, 6, 7, 8, 9, 0, 1, 2, 3, 4, 5, 6, 7, 8,
                         9]
        year_digits = 'ABCDEFGHJKLMNPRSTVWXY123456789'
        vin = ''

        temp_vin_maker_code = self._vin_makers[automaker]
        vin += random.choice(temp_vin_maker_code)  # 1-3 digits
        vin += ''.join(random.choices(letters_nb, k=6))  # 4-9 digits
        vin += year_digits[(int(production_year) - 1980) % len(year_digits)]  # 10 digit
        vin += random.choice(letters)  # 11 digit
        production_nb = str(randrange(1, 99999))
        vin += production_nb.rjust(5, '0')  # 12-16 digits
        check_sum = nb_for_letter[letters_nb.find(vin[0])] * 9 + nb_for_letter[letters_nb.find(vin[1])] * 8 \
                    + nb_for_letter[letters_nb.find(vin[2])] * 7 + nb_for_letter[letters_nb.find(vin[3])] * 6 \
                    + nb_for_letter[letters_nb.find(vin[4])] * 5 + nb_for_letter[letters_nb.find(vin[5])] * 4 \
                    + nb_for_letter[letters_nb.find(vin[6])] * 3 + nb_for_letter[letters_nb.find(vin[7])] * 2 \
                    + nb_for_letter[letters_nb.find(vin[8])] * 10 + nb_for_letter[letters_nb.find(vin[9])] * 9 \
                    + nb_for_letter[letters_nb.find(vin[10])] * 8 + nb_for_letter[letters_nb.find(vin[11])] * 7 \
                    + nb_for_letter[letters_nb.find(vin[12])] * 6 + nb_for_letter[letters_nb.find(vin[13])] * 5 \
                    + nb_for_letter[letters_nb.find(vin[14])] * 4 + nb_for_letter[letters_nb.find(vin[15])] * 3
        for i in range(0, 9):
            if (check_sum + (i * 2)) % 11 == i:
                vin += str(i)
                return vin

    def generate(self, lang: str = 'PL', count: int = 1):
        """Generating vehicles information

        Parameters:
        lang (str): Two letter shortcut for country name, default is 'PL'
        count (int):  Number how many complete information of vehicles should be generated

        Return:
        json:  Returns data in json format
        """
        generated_cars = self._car_generator.generate(lang=lang, data_type='cars', counter=count, sep='\t')

        for i in range(count):
            model = self._toss_a_model(generated_cars[i])
            self._car[i] = {
                'brand': generated_cars[i],
                'model': model[0],
                'type': model[1],
                'body': self._body_type[model[2]],
                'origin': model[3],
                'year_of_production': model[4],
                'engine_capacity': int(model[5].split('.')[0]),
                'engine_power': int(model[6].split('.')[0]),
                'nb_of_seats': model[7],
                'fuel_type': model[8],
                'vin': self._get_vin(generated_cars[i], model[4])
            }
        return json.dumps(self._car, indent=4, ensure_ascii=False)



# Always prefer setuptools over distutils
from setuptools import setup
# To use a consistent encoding
from codecs import open
from os import path

# The directory containing this file
HERE = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(HERE, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()


setup(
    name='baraqda-lib',
    version='0.0.16',
    packages=['tests', 'baraqdalib'],
    url='',
    license='MIT',
    author='Kacper Hanuszewicz, Aleksander Karbowy, Wojciech Marcinkowski, Marcin Rogoń',
    author_email='',
    description='Generator of real fake data for testing purpose using real distribution in society',
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Operating System :: OS Independent'
    ],
    include_package_data=True,
    install_requires=[]
)
